@@ -1,11 +1,11 @@
 package ZMQ::FFI::ContextRole;
-$ZMQ::FFI::ContextRole::VERSION = '1.08';
+$ZMQ::FFI::ContextRole::VERSION = '1.09';
 use Moo::Role;
 
 use ZMQ::FFI::Util qw(current_tid);
 
-# real underlying zmq ctx pointer
-has _ctx => (
+# real underlying zmq context pointer
+has context_ptr => (
     is      => 'rw',
     default => -1,
 );
@@ -37,6 +37,12 @@ has max_sockets => (
     predicate => 'has_max_sockets',
 );
 
+has sockets => (
+    is        => 'rw',
+    lazy      => 1,
+    default   => sub { [] },
+);
+
 requires qw(
     get
     set
@@ -60,7 +66,7 @@ ZMQ::FFI::ContextRole
 
 =head1 VERSION
 
-version 1.08
+version 1.09
 
 =head1 AUTHOR
 
